@@ -4,3 +4,8 @@ class PrivateClass(object):
     def public_method():
         """This is public."""
         return 5
+
+
+def imported_function():
+    """A function that gets imported."""
+    return 1
