@@ -4,7 +4,7 @@ This is a description
 """
 
 from ._private_module import PrivateClass as PublicClass
-from ._subpackage import module_level_method
+from ._subpackage import module_level_function
 
 __all__ = ["PublicClass", "Foo"]
 
