@@ -1,19 +1,17 @@
 import io
 import os
 import pathlib
-import shutil
 import sys
 from unittest.mock import Mock, call
 
 import autoapi.settings
-from autoapi.mappers.python import (
+from autoapi._objects import (
     PythonClass,
     PythonData,
     PythonFunction,
     PythonMethod,
     PythonModule,
 )
-from bs4 import BeautifulSoup
 from packaging import version
 import pytest
 import sphinx
@@ -24,60 +22,6 @@ import sphinx.util.logging
 sphinx_version = version.parse(sphinx.__version__).release
 
 
-def rebuild(confdir=".", **kwargs):
-    app = Sphinx(
-        srcdir=".",
-        confdir=confdir,
-        outdir="_build/html",
-        doctreedir="_build/.doctrees",
-        buildername="html",
-        **kwargs,
-    )
-    app.build()
-
-
-@pytest.fixture(scope="class")
-def builder():
-    cwd = os.getcwd()
-
-    def build(test_dir, **kwargs):
-        if kwargs.get("warningiserror"):
-            # Add any warnings raised when using `Sphinx` more than once
-            # in a Python session.
-            confoverrides = kwargs.setdefault("confoverrides", {})
-            confoverrides.setdefault("suppress_warnings", [])
-            suppress = confoverrides["suppress_warnings"]
-            suppress.append("app.add_node")
-            suppress.append("app.add_directive")
-            suppress.append("app.add_role")
-
-        os.chdir("tests/python/{0}".format(test_dir))
-        rebuild(**kwargs)
-
-    yield build
-
-    try:
-        shutil.rmtree("_build")
-        if (pathlib.Path("autoapi") / "index.rst").exists():
-            shutil.rmtree("autoapi")
-    finally:
-        os.chdir(cwd)
-
-
-@pytest.fixture(scope="class")
-def parse():
-    cache = {}
-
-    def parser(path):
-        if path not in cache:
-            with io.open(path, encoding="utf8") as file_handle:
-                cache[path] = BeautifulSoup(file_handle, features="html.parser")
-
-        return cache[path]
-
-    yield parser
-
-
 class TestSimpleModule:
     @pytest.fixture(autouse=True, scope="class")
     def built(self, builder):
@@ -200,7 +144,7 @@ class TestSimpleModule:
     def test_long_signature(self, parse):
         example_file = parse("_build/html/autoapi/example/index.html")
 
-        summary_row = example_file.find_all(class_="autosummary")[1].find_all("tr")[-1]
+        summary_row = example_file.find_all(class_="autosummary")[-1].find_all("tr")[-1]
         assert summary_row
         cells = summary_row.find_all("td")
         assert (
@@ -300,6 +244,20 @@ class TestSimpleStubModuleNotPreferred:
         assert foo_sig
 
 
+class TestStubInitModuleInSubmodule:
+    @pytest.fixture(autouse=True, scope="class")
+    def built(self, builder):
+        builder("pyisubmoduleinit", warningiserror=True)
+
+    def test_integration(self, parse):
+        example_file = parse("_build/html/autoapi/example/index.html")
+
+        # Documentation should list
+        # submodule_foo instead of __init__
+        assert example_file.find(title="submodule_foo")
+        assert not example_file.find(title="__init__")
+
+
 class TestPy3Module:
     @pytest.fixture(autouse=True, scope="class")
     def built(self, builder):
@@ -629,6 +587,34 @@ class TestPipeUnionModule:
         assert links[1].text == "None"
 
 
+@pytest.mark.skipif(
+    sys.version_info < (3, 12), reason="PEP-695 support requires Python >=3.12"
+)
+class TestPEP695:
+    @pytest.fixture(autouse=True, scope="class")
+    def built(self, builder):
+        builder("pep695", warningiserror=True)
+
+    def test_integration(self, parse):
+        example_file = parse("_build/html/autoapi/example/index.html")
+
+        alias = example_file.find(id="example.MyTypeAliasA")
+        properties = alias.find_all(class_="property")
+        assert len(properties) == 2
+        annotation = properties[0].text
+        assert annotation == ": TypeAlias"
+        value = properties[1].text
+        assert value == " = tuple[str, int]"
+
+        alias = example_file.find(id="example.MyTypeAliasB")
+        properties = alias.find_all(class_="property")
+        assert len(properties) == 2
+        annotation = properties[0].text
+        assert annotation == ": TypeAlias"
+        value = properties[1].text
+        assert value == " = tuple[str, int]"
+
+
 def test_napoleon_integration_loaded(builder, parse):
     confoverrides = {
         "exclude_patterns": ["manualapi.rst"],
@@ -656,26 +642,26 @@ class TestSimplePackage:
         builder("pypackageexample", warningiserror=True)
 
     def test_integration_with_package(self, parse):
-        example_file = parse("_build/html/autoapi/example/index.html")
+        example_file = parse("_build/html/autoapi/package/index.html")
 
         entries = example_file.find_all(class_="toctree-l1")
-        assert any(entry.text == "example.foo" for entry in entries)
-        assert example_file.find(id="example.module_level_method")
+        assert any(entry.text == "package.submodule" for entry in entries)
+        assert example_file.find(id="package.function")
 
-        example_foo_file = parse("_build/html/autoapi/example/foo/index.html")
+        example_foo_file = parse("_build/html/autoapi/package/submodule/index.html")
 
-        foo = example_foo_file.find(id="example.foo.Foo")
-        assert foo
-        method_okay = foo.parent.find(id="example.foo.Foo.method_okay")
+        submodule = example_foo_file.find(id="package.submodule.Class")
+        assert submodule
+        method_okay = submodule.parent.find(id="package.submodule.Class.method_okay")
         assert method_okay
 
         index_file = parse("_build/html/index.html")
 
         toctree = index_file.select("li > a")
         assert any(item.text == "API Reference" for item in toctree)
-        assert any(item.text == "example.foo" for item in toctree)
-        assert any(item.text == "Foo" for item in toctree)
-        assert any(item.text == "module_level_method()" for item in toctree)
+        assert any(item.text == "package.submodule" for item in toctree)
+        assert any(item.text == "Class" for item in toctree)
+        assert any(item.text == "function()" for item in toctree)
 
 
 def test_simple_no_false_warnings(builder, caplog):
@@ -724,14 +710,14 @@ def test_hiding_private_members(builder, parse):
     confoverrides = {"autoapi_options": ["members", "undoc-members", "special-members"]}
     builder("pypackageexample", warningiserror=True, confoverrides=confoverrides)
 
-    example_file = parse("_build/html/autoapi/example/index.html")
+    example_file = parse("_build/html/autoapi/package/index.html")
 
     entries = example_file.find_all(class_="toctree-l1")
     assert all("private" not in entry.text for entry in entries)
 
-    private_file = parse("_build/html/autoapi/example/_private_module/index.html")
-
-    assert private_file.find(id="example._private_module.PrivateClass.public_method")
+    assert not pathlib.Path(
+        "_build/html/autoapi/package/_private_module/index.html"
+    ).exists()
 
 
 def test_hiding_inheritance(builder, parse):
@@ -760,6 +746,16 @@ def test_hiding_imported_members(builder, parse):
     assert not submodule_file.find(id="complex.subpackage.now_public_function")
 
 
+def test_imports_into_modules_always_hidden(builder, parse):
+    confoverrides = {
+        "autoapi_options": ["members", "undoc-members", "imported-members"]
+    }
+    builder("pypackagecomplex", confoverrides=confoverrides)
+
+    submodule_file = parse("_build/html/autoapi/complex/submodule/index.html")
+    assert not submodule_file.find(id="complex.submodule.imported_function")
+
+
 def test_inherited_members(builder, parse):
     confoverrides = {
         "autoapi_options": ["members", "inherited-members", "undoc-members"],
@@ -963,7 +959,7 @@ class TestComplexPackage:
         assert wildcard_file.find(id="complex.wildcard.public_chain")
         assert wildcard_file.find(id="complex.wildcard.now_public_function")
         assert wildcard_file.find(id="complex.wildcard.public_multiple_imports")
-        assert wildcard_file.find(id="complex.wildcard.module_level_method")
+        assert wildcard_file.find(id="complex.wildcard.module_level_function")
 
     def test_wildcard_all_imports(self, parse):
         wildcard_file = parse("_build/html/autoapi/complex/wildall/index.html")
@@ -974,12 +970,12 @@ class TestComplexPackage:
         assert wildcard_file.find(id="complex.wildall.SimpleClass")
         assert wildcard_file.find(id="complex.wildall.simple_function")
         assert wildcard_file.find(id="complex.wildall.public_chain")
-        assert wildcard_file.find(id="complex.wildall.module_level_method")
+        assert wildcard_file.find(id="complex.wildall.module_level_function")
 
     def test_no_imports_in_module_with_all(self, parse):
         foo_file = parse("_build/html/autoapi/complex/foo/index.html")
 
-        assert not foo_file.find(id="complex.foo.module_level_method")
+        assert not foo_file.find(id="complex.foo.module_level_function")
 
     def test_all_overrides_import_in_module_with_all(self, parse):
         foo_file = parse("_build/html/autoapi/complex/foo/index.html")
@@ -991,6 +987,13 @@ class TestComplexPackage:
 
         assert foo_file.find(id="complex.unicode_data.unicode_str")
 
+    def test_nested_parse_directive(self, parse):
+        package_file = parse("_build/html/autoapi/complex/index.html")
+
+        complex = package_file.find(id="complex")
+        assert "This heading will be removed" not in complex.parent.text
+        assert complex.parent.find("section")["id"] != "this-heading-will-be-removed"
+
 
 class TestComplexPackageParallel(TestComplexPackage):
     @pytest.fixture(autouse=True, scope="class")
@@ -998,7 +1001,7 @@ class TestComplexPackageParallel(TestComplexPackage):
         builder("pypackagecomplex", parallel=2)
 
 
-def test_caching(builder):
+def test_caching(builder, rebuild):
     mtimes = (0, 0)
 
     def record_mtime():
@@ -1104,25 +1107,25 @@ def test_string_module_attributes(builder):
         ".. py:data:: code_snippet",
         "   :value: Multiline-String",
         "",
-        "    .. raw:: html",
+        "   .. raw:: html",
         "",
-        "        <details><summary>Show Value</summary>",
+        "      <details><summary>Show Value</summary>",
         "",
-        "    .. code-block:: python",
+        "   .. code-block:: python",
         "",
-        '        """The following is some code:',
-        "        ",  # <--- Line array monstrosity to preserve these leading spaces
-        "        # -*- coding: utf-8 -*-",
-        "        from __future__ import absolute_import, division, print_function, unicode_literals",
-        "        # from future.builtins.disabled import *",
-        "        # from builtins import *",
-        "        ",
-        """        print("chunky o'block")""",
-        '        """',
+        '      """The following is some code:',
+        "      ",  # <--- Line array monstrosity to preserve these leading spaces
+        "      # -*- coding: utf-8 -*-",
+        "      from __future__ import absolute_import, division, print_function, unicode_literals",
+        "      # from future.builtins.disabled import *",
+        "      # from builtins import *",
+        "      ",
+        """      print("chunky o'block")""",
+        '      """',
         "",
-        "    .. raw:: html",
+        "   .. raw:: html",
         "",
-        "        </details>",
+        "      </details>",
     ]
     assert "\n".join(code_snippet_contents) in example_file
 
